/// Centroid Decomposition
///
/// Description:
///   The Centroid Decomposition of a tree is a tree where: 1) its root is the
/// centroid of the original tree, and 2) its children are the centroid of each
/// tree resulting from the removal of the root from the original tree. \par
///   The result is a tree with $\log n$ height, where the path from $a$ to $b$,
/// in the original tree, can be decomposed into the path from a to lca($a$,$b$)
/// and from lca($a$,$b$) to $b$. \par
///   This is useful because each one of the $n^2$ paths of the original tree
/// is a concatenation of two paths in a set of $O(n \log n)$ paths (from each
/// node to all of its ancestors in the centroid decomposition).
///
/// Time: O(V log V)
/// Space: O(V + E)

// Must be a tree
vector<int> graph[MAX];

struct CentroidDecomposition {
    vector<int> par, size, vis;

    CentroidDecomposition(int N) :
        par(N), size(N), vis(N) 
    { init(); }

    void init() {
        fill(all(vis), 0);
        build(0); // 0-indexed vertices
    }

    void build(int x, int p = -1) {
        int n = dfs(x);
        int c = get_centroid(x, n);
        vis[c] = 1;
        par[c] = p;

        for (auto i : graph[c])
            if (!vis[i])
                build(i, c);
    }

    // Calculates size of every subtree.
    int dfs(int x, int p = -1) {
        size[x] = 1;
        for (auto i : graph[x])
            if (i != p && !vis[i])
                size[x] += dfs(i, x);
        return size[x];
    }

    int get_centroid(int x, int n, int p = -1) {
        for (auto i : graph[x])
            if (i != p && size[i] > n / 2 && !vis[i])
                return get_centroid(i, n, x);
        return x;
    }

    int operator[](int i) { return par[i]; }
};
