/// Heavy Light Decomposition
///
/// Description:
///   The Heavy Light Decomposition splits a tree into several path so that we
/// can reach the root from any node by traversing at most $\log n$ paths. For
/// every vertice $v$, an edge is called heavy if it leads to a child with the
/// largest subtree size; all other edges are called light. \par
///   The set of disjoint paths containing heavy edges are called the ``heavy
/// chains''. These paths (both heavy and light) can be indexed by a single 
/// segment tree, allowing fast queries and updates to edges (or vertices). And
/// also, information about the chains (such as the head), allows for fast 
/// traversals from any node to any of its ancestors.
///
/// Time: 
///   - build: O(n log n)
///   - query: O(log^2 n)
///   - update: O(log n)
/// Space: O(n log n)
/// 
/// Caution:
///   - Modifications are necessary if values are associated with vertices;
///   - Modifications are necerrary if other data structure is used.
/// 
/// Status: Tested (QTREE,URI2887)

p<int> edge[MAX];
vector<p<int>> graph[MAX];

template <typename ST>
struct HLD {
    ST &seg;
    int cnum, ptr;

    // depth, parent, value of node
    vector<int> dep, par, val;

    // head[i]: head of i-th node's chain;
    // heavy[i]: "special child" of i-th node
    // pos[i]: position of i-th node on segtree
    // bot[i]: bottommost (depth-wise) node on the i-th 
    //   edge (required only when edges have weights)
    vector<int> head, heavy, pos, bot;

    HLD(int n, ST &seg) :
        seg(seg), 
        dep(n, 0), par(n), val(n),
        head(n), heavy(n, -1), pos(n), bot(n)
    {
        cnum = ptr = 0;
        N = n; // global N for segtree
        dfs(0);
        decompose(0);

        // (required only when edges have weights)
        for (int i = 0; i < n - 1; ++i)
            if (dep[edge[i].fi] > dep[edge[i].se])
                bot[i] = edge[i].fi;
            else
                bot[i] = edge[i].se;
    }

    int dfs(int x, int p = -1) {
        int size = 1;
        par[x] = p;

        int max_size = 0;
        for (auto i : graph[x])
            if (i.fi != p) {
                dep[i.fi] = dep[x] + 1;
                val[i.fi] = i.se;
                int isize = dfs(i.fi, x);

                size += isize;
                if (isize > max_size)
                    max_size = isize, heavy[x] = i.fi;
            }

        return size;
    }

    void decompose(int x, int h = 0) {
        head[x] = h;
        seg.update(ptr, val[x]);
        pos[x] = ptr++;

        if (heavy[x] != -1)
            decompose(heavy[x], h);

        for (auto i : graph[x])
            if (i.fi != par[x] && i.fi != heavy[x])
                decompose(i.fi, i.fi);
    }

    // Queries max edge (or vertice) on the path 
    // between a and b
    int query(int a, int b) {
        int ans = seg.ident;
        for (; head[a] != head[b]; b = par[head[b]]) {
            if (dep[head[a]] > dep[head[b]])
                swap(a, b);
            ans = seg.func(ans, seg.query(pos[head[b]], pos[b]));
        }

        if (dep[a] > dep[b]) swap(a, b);

        // Remove "+ 1" when values are associated with vertices
        return seg.func(ans, seg.query(pos[a] + 1, pos[b]));
    }

    // Updates value of i-th edge (or vertice)
    void update(int i, int val) {
        seg.update(pos[bot[i]], val);
    }
};
