/// Lowest Common Ancestor (LCA)
///
/// Description:
///   The LCA between two nodes in a tree is a node that is an ancestor to both
/// nodes with the lowest height possible. \par
///   The algorithm works by following the path up the tree from both nodes 
/// ``simultaneously'' until a common node is found. The naive approach for 
/// that would be $O(n)$ in the worst case. To improve that, this 
/// implementation uses ``binary lifting'' which is a way of figuring out the 
/// right number of up-moves needed to find the LCA by following the binary 
/// representation of the distance to the destination (similar to the ``binary 
/// search by jumping''), but, for that, a preprocessing must be done to set
/// every parent at a $2^i$ distance.
///
/// Time:
///   - preprocess: O(V log V)
///   - query:      O(log V)
/// Space: O(V + E + V log V)

#define LOG 20
#define COST 0

vector<p<int>> graph[MAX];
int par[MAX][LOG], cost[MAX][LOG];

template <typename T, class F = function<T(T,T)>>
struct LCA {
    F func;
    vector<int> h;

    LCA(int N, F func) : h(N), func(func)
    { init(); }

    void init() {
        mset(par, -1);
        mset(cost, 0);
        dfs(0);
    }

    void dfs(int v, int p = -1, int c = 0) {
        par[v][0] = p;
        cost[v][0] = c;
        if (p != -1) h[v] = h[p] + 1;

        for (int i = 1; i < LOG; ++i)
            if (par[v][i-1] != -1) {
                par[v][i] = par[par[v][i-1]][i-1];
                cost[v][i] = func(cost[v][i], func(cost[v][i-1],
                            cost[par[v][i-1]][i-1]));
            }

        for (auto u : graph[v]) 
            if (p != u.fi)
                dfs(u.fi, v, u.se);
    }

    int query(int a, int b) {
        int ans = 0;
        if (h[a] < h[b]) swap(a, b);

        for (int i = LOG - 1; i >= 0; --i)
            if (par[a][i] != -1 && h[par[a][i]] >= h[b]) {
                ans = func(ans, cost[a][i]);
                a = par[a][i];
            }

        if (a == b) {
#ifdef COST
            return ans;
#else
            return a;
#endif
        }

        for (int i = LOG - 1; i >= 0; --i)
            if (par[a][i] != -1 && par[a][i] != par[b][i]) {
                ans = func(ans, func(cost[a][i], cost[b][i]));
                a = par[a][i];
                b = par[b][i];
            }

#ifdef COST
        if (a == b) 
            return ans;
        else 
            return func(ans, func(cost[a][0], cost[b][0]));
#else
        return par[a][0];
#endif
    }
};
