/// Steiner Tree
///
/// Description:
///   A (minimum) Steiner tree is a tree that, given a graph G and a set of
/// terminal vertices T (inside the graph), connects all vertices in T using 
/// other vertices in G if necessary, ans minimizes the sum of weights of the 
/// included edges. \par
///   The algorithm uses dynamic programming, where dp[i][mask] 
/// stores the value of a Steiner tree rooted at $i$ containing the terminal
/// nodes represented by the bits in $mask$. The algorithm iterates over all
/// bitmasks, and, at each iteration $mask$, every pair of complementary subsets
/// are tested and dp[i][mask] is updated with the value given by the 
/// combination of both trees. Then, still at step $mask$, dp[j][mask] is 
/// updated for every $j$ with the ``extension'' of $i$ (for every $i$), as if 
/// the root was moving around. \par
///   The result must be retrieved from a node (root of the final Steiner tree) 
/// that contains all terminal nodes and has the smallest value.
///
/// Time: O(n^2 * 2^t + n * 3^t)
/// Space: O(n * 2^t + n^2)
///
/// Status: Tested (URI2885)

int dist[MAXN][MAXN];
int graph[MAXN][MAXN];
int dp[MAXN][1 << MAXT];

int steiner_tree(int n, int t) {
    floyd_warshall(n);

    mset(dp, inf);
    for (int i = 0; i < t; ++i)
        dp[i][1 << i] = 0;

    for (int mask = 1; mask < (1 << t); ++mask) {
        for (int i = 0; i < n; ++i)
            for (int ss = mask; ss > 0; ss = (ss - 1) & mask)
                dp[i][mask] = min(dp[i][mask], 
                        dp[i][ss] + dp[i][mask ^ ss]);

        for (int i = 0; i < n; ++i)
            for (int j = 0; j < n; ++j)
                dp[j][mask] = min(dp[j][mask], 
                        dp[i][mask] + dist[i][j]);
    }

    int ans = inf;
    for (int i = 0; i < n; ++i)
        ans = min(ans, dp[i][(1 << t) - 1]);
    return ans;
}
