/// Chinese Remainder Theorem
///
/// Description:
///   Given $t$ linear congruences in the format: \[ x \equiv a_i \pmod{m_i} \]
/// the Chinese Remainder Theorem (CRT) finds $x$ that satisfies every 
/// given congruence or states that there are no solutions. \par
///   This implementation does not require all $m_i$ to be coprime, it works
/// with any value and returns the smallest possible $x$; infinitely many 
/// solutions can be obtained by incrementing or decrementing 
/// $LCM(m_1,m_2,...,m_t)$ from $x$.
///
/// Time: O(t log LCM(m_1,m_2,...,m_t))
/// Space: O(t)
///
/// Caution:
///   - It is very easy to get overflow, since the $LCM$ is computed from
/// all $m_i$, BigInt or Python should be considered if inputs are too large

i64 norm(i64 a, i64 b) { 
    a %= b; 
    return (a < 0) ? a + b : a;
}

pair<i64,i64> crt_single(i64 a, i64 n, i64 b, i64 m) {
    ans_t e = ext_gcd(n, m);

    if ((a - b) % e.d != 0)
        return {-1,-1}; // No solution

    i64 lcm = (m/e.d) * n;
    i64 ans = norm(a + e.x*(b-a) / e.d % (m/e.d)*n, lcm);
    return {norm(ans, lcm), lcm};
}

i64 crt(vector<i64> a, vector<i64> m) {
    i64 ans = a[0];
    i64 lcm = m[0];

    int t = a.size();
    for (int i = 1; i < t; ++i) {
        auto ss = crt_single(ans, lcm, a[i], m[i]);
        if (ss.fi == -1)
            return -1; // No solution

        ans = ss.fi;
        lcm = ss.se;
    }

    return ans;
}
