/// Fast Fourier Transform (FFT)
///
/// Time: O(N log N)
/// Space: O(N)

struct FFT {
    struct Complex {
        float r, i;

        Complex() : r(0), i(0) {}
        Complex(float r, float i) : r(r), i(i) {}

        Complex operator+(Complex b) { 
            return Complex(r + b.r, i + b.i); 
        }

        Complex operator-(Complex b) { 
            return Complex(r - b.r, i - b.i); 
        }

        Complex operator*(Complex b) { 
            return Complex(r*b.r - i*b.i, r*b.i + i*b.r); 
        }

        Complex operator/(Complex b) {
            float div = (b.r * b.r) + (b.i * b.i);
            return Complex((r * b.r + i * b.i) / div, 
                    (i * b.r - r * b.i) / div);
        }

        static inline Complex conj(Complex a) { 
            return Complex(a.r, -a.i); 
        }
    };

    vector<int> rev = {0, 1};
    vector<Complex> roots = {{0, 0}, {1, 0}};

    // Initializes reversed-bit vector (rev) and 
    // roots of unity vector (roots)
    void init(int nbase) {
        rev.resize(1 << nbase);
        roots.resize(1 << nbase);

        // Build rev vector
        for (int i = 0; i < (1 << nbase); ++i)
            rev[i] = (rev[i >> 1] >> 1) + \
                     ((i & 1) << (nbase - 1));

        // Build roots vector
        for (int base = 1; base < nbase; ++base) {
            float angle = 2 * M_PI / (1 << (base + 1));

            for (int i = 1 << (base - 1); i < (1 << base); ++i) {
                float angle_i = angle * (2*i + 1 - (1 << base));

                roots[i << 1] = roots[i];
                roots[(i << 1) + 1] = Complex(cos(angle_i),
                        sin(angle_i));
            }
        }
    }

    void fft(vector<Complex> &a) {
        int n = a.size();

        for (int i = 0; i < n; ++i)
            if (i < rev[i])
                swap(a[i], a[rev[i]]);

        for (int s = 1; s < n; s <<= 1) {
            for (int k = 0; k < n; k += (s << 1)) {
                for (int j = 0; j < s; ++j) {
                    Complex z = a[k + j + s] * roots[j + s];
                    a[k + j + s] = a[k + j] - z;
                    a[k + j] = a[k + j] + z;
                }
            }
        }
    }

    vector<int> multiply(const vector<int> &a, 
            const vector<int> &b) 
    {
        int nbase, need = a.size() + b.size() + 1;

        for (nbase = 0; (1 << nbase) < need; ++nbase);
        init(nbase);

        int size = 1 << nbase;
        vector<Complex> fa(size);

        for (int i = 0; i < size; ++i) {
            int x = (i < a.size() ? a[i] : 0);
            int y = (i < b.size() ? b[i] : 0);
            fa[i] = Complex(x, y);
        }

        fft(fa);

        Complex r(0, -0.25 / size);
        for (int i = 0; i <= (size >> 1); ++i) {
            int j = (size - i) & (size - 1);
            Complex z = (fa[j]*fa[j] - conj(fa[i]*fa[i])) * r;

            if (i != j)
                fa[j] = (fa[i]*fa[i] - conj(fa[j]*fa[j])) * r;

            fa[i] = z;
        }

        fft(fa);

        vector<int> res(need);
        for (int i = 0; i < need; ++i)
            res[i] = fa[i].r + 0.5;

        return res;
    }
};
