/// Gale-Shapley (Stable Marriage)
///
/// Description:
///   Two groups, each of size N are given: men and women. Each person has a
/// list of preference ranking all N people of the opposite sex. The task is to
/// unite both groups into stable pairs. A set of pairs is stable if there are
/// no unassigned couple that like each other more than their assigned pair.\par
///   The algorithm's steps are: 1) allow every men to propose to their highest
/// ranking woman; 2) the women become tentatively engaged to their top choice
/// of men; 3) all rejected men propose to their next choice; 4) the woman
/// replaces their current pair in case a man with a higher rank proposes to
/// her, the replaced men are now marked as rejected; 5) repeat step 3 until
/// all men are paired. \par
///   The result is garanteed to return a configuration where every man gets
/// the best possible wife, while every woman gets the worst possible husband
/// (it benefits the group that chooses first).
///
/// Time: O(n^2)
/// Space: O(n^2)
///
/// Caution:
///   - Men are indexed by $[0,N)$
///   - Women are indexed by $[N,2N)$
///   - The result prioritizes men, swapping the bottom half
///   of the matrix by the top half will invert who's given
///   preference
///
/// Status: Tested (URI1852)

// Receives matrix of preferences pref[2*N][N] and returns
// vector v where v[m] contains preference of the m-th man.
vector<int> gale_shapley(const vector<vector<int>> &pref) {
    int n = pref[0].size();
    vector<int> w_part(n, -1);
    vector<int> m_part(n, -1);
    vector<int> start(n, 0);

    while (true) {
        int m;
        for (m = 0; m < n; ++m)
            if (m_part[m] == -1)
                break;

        if (m == n) break;

        for (; start[m] < n && m_part[m] == -1; ++start[m]) {
            int w = pref[m][start[m]];

            if (w_part[w - n] == -1) {
                w_part[w - n] = m;
                m_part[m] = w;
            } else {
                int m1 = w_part[w - n];
                bool pref_m = false;

                for (int j = 0; j < n; ++j)
                    if (pref[w][j] == m) {
                        pref_m = true;
                        break;
                    } else if (pref[w][j] == m1)
                        break;

                if (pref_m) {
                    w_part[w - n] = m;
                    m_part[m] = w;
                    m_part[m1] = -1;
                }
            }
        }
    }

    return m_part;
}
