/// Linear Diophantine Equation
///
/// Description:
///   A Linear Diophantine Equation is an equation in the form \[ ax + by = c \]
/// A solution of this equation is a pair $(x,y)$ that satisfies the equation. 
/// The locus of (lattice) points whose coordinates $x$ and $y$ satisfy the 
/// equation is a straigh line. \par
///   The equation has a solution only if $gcd(a,b) | c$. In the case of 
/// existing a solution for the provided $a, b, c$, the infinite set of
/// coordinates $(x,y)$ can be obtained with get(t) for 
/// $t=...,-2,-1,0,1,2,...$.
///
/// Time: O(log min(a,b))
/// Space: O(1)

struct Diophantine {
    int a, b, c, d;
    int x0, y0;

    bool has_solution;

    Diophantine(int a, int b, int c) :
        a(a), b(b), c(c) 
    { init(); }

    void init() {
        ans_t e = ext_gcd(a, b); d = e.d;
        if (c % d == 0) {
            x0 = e.x * (c / d);
            y0 = e.y * (c / d);
            has_solution = true;
        } else
            has_solution = false;
    }

    p<int> get(int t) {
        if (!has_solution) return p<int>(inf, inf);
        return p<int>(x0 + t * (b / d), y0 - t * (a / d));
    }
};
