/// Lazy Segment Tree
///
/// Time:
///   - build:  O(n log n)
///   - update: O(log n)
///   - query:  O(log n)
/// Space: O(n)
///
/// Caution:
///   - Provide value for N before any operation
///   - Provide identity value if necessary (default is T())

#define left(x) (x << 1)
#define right(x) ((x << 1) + 1)

int N;

template <typename T, class F = function<T(const T&, const T&)>>
struct LazySegmentTree {
    F func;
    T id = T();
    vector<T> tree, lazy;

    LazySegmentTree(F func) : 
        func(func), tree(MAX*4, 0), lazy(MAX*4, 0) {}

    void build(const vector<T> &v, 
            int node = 1, int l = 0, int r = N - 1) 
    {
        if (l > r) return;

        if (l == r)
            tree[node] = v[l];
        else {
            int m = (l + r) / 2;
            build(v, left(node), l, m);
            build(v, right(node), m + 1, r);
            tree[node] = func(tree[left(node)], tree[right(node)]);
        }
    }

    void push(int node, int l, int r, T val) {
        tree[node] += (r - l + 1) * val;

        if (l != r) {
            lazy[left(node)] += val;
            lazy[right(node)] += val;
        }

        lazy[node] = 0;
    }

    void update(int i, int j, T val, 
            int node = 1, int l = 0, int r = N - 1) 
    {
        if (lazy[node] != 0)
            push(node, l, r, lazy[node]);

        if (l > r || l > j || r < i) return;

        if (i <= l && r <= j)
            push(node, l, r, val);
        else {
            int m = (l + r) / 2;
            update(i, j, val, left(node), l, m);
            update(i, j, val, right(node), m + 1, r);
            tree[node] = func(tree[left(node)], tree[right(node)]);
        }
    }

    T query(int i, int j, 
            int node = 1, int l = 0, int r = N - 1)
    {
        if (l > r || l > j || r < i) return id;

        if (lazy[node])
            push(node, l, r, lazy[node]);

        if (l >= i && r <= j)
            return tree[node];

        int m = (l + r) / 2;
        T q1 = query(i, j, left(node), l, m);
        T q2 = query(i, j, right(node), m + 1, r);
        return func(q1, q2);
    }
};
