/// 2D Segment Tree
///
/// Time:
///   - build:  O(n * m * log n * log m)
///   - update: O(log n * log m)
///   - query:  O(log n * log m)
/// Space: O(16 * n * m)
///
/// Caution:
///   - Very high constant in space complexity

#define left(x) (x << 1)
#define right(x) ((x << 1) + 1)

int N, M;

template<typename T, class F = function<T(T,T)>>
struct SegmentTree2D {
    using matrix<T> = vector<vector<T>>;

    F func;
    T id = T();
    matrix<T> tree;

    SegmentTree2D(F func) :
        tree(4*MAX, vector<T>(4*MAX, 0)),
        func(func) {}

    void build_row(const matrix<T> &mat, 
            int ni, int li, int ri,
            int nj = 1, int lj = 0, int rj = M - 1)
    {
        if (lj == rj) {
            if (li == ri) 
                tree[ni][nj] = mat[li][lj];
            else 
                tree[ni][nj] = func(tree[left(ni)][nj], tree[right(ni)][nj]);
        } else {
            int m = (lj + rj) / 2;
            build_row(mat, ni, li, ri, left(nj), lj, m);
            build_row(mat, ni, li, ri, right(nj), m+1, rj);
            tree[ni][nj] = func(tree[ni][left(nj)], tree[ni][right(nj)]);
        }
    }

    void build(const matrix<T> &mat,
            int ni = 1, int li = 0, int ri = N - 1) 
    {
        if (li != ri) {
            int m = (li + ri) / 2;
            build(mat, left(ni), li, m);
            build(mat, right(ni), m+1, ri);
        }
        build_row(mat, ni, li, ri);
    }

    T query_row(int j1, int j2, int i,
            int nj = 1, int lj = 0, int rj = M - 1) 
    {
        if (lj > rj || lj > j2 || rj < j1) return id;

        if (j1 <= lj && rj <= j2)
            return tree[i][nj];

        int m = (lj + rj) / 2;
        T q1 = query_row(j1, j2, i, left(nj), lj, m);
        T q2 = query_row(j1, j2, i, right(nj), m + 1, rj);
        return func(q1, q2);
    }

    T query(int i1, int j1, int i2, int j2,
            int ni = 1, int li = 0, int ri = N - 1) 
    {
        if (li > ri || li > i2 || ri < i1) return id;

        if (i1 <= li && ri <= i2)
            return query_row(j1, j2, ni);

        int m = (li + ri) / 2;
        T q1 = query(i1, j1, i2, j2, left(ni), li, m);
        T q2 = query(i1, j1, i2, j2, right(ni), m + 1, ri);
        return func(q1, q2);
    }

    void update_row(int i, int j, T val,
            int ni, int li, int ri,
            int nj = 1, int lj = 0, int rj = M - 1)
    {
        if (lj > rj || lj > j || rj < j) return;

        if (lj == rj) {
            if (li == ri) 
                tree[ni][nj] = val;
            else 
                tree[ni][nj] = func(tree[left(ni)][nj], tree[right(ni)][nj]);
        } else {
            int m = (lj + rj) / 2;
            update_row(i, j, val, ni, li, ri, left(nj), lj, m);
            update_row(i, j, val, ni, li, ri, right(nj), m+1, rj);
            tree[ni][nj] = func(tree[ni][left(nj)], tree[ni][right(nj)]);
        }
    }

    void update(int i, int j, T val, 
            int ni = 1, int li = 0, int ri = N - 1) 
    {
        if (li > ri || li > i || ri < i) return;

        if (li != ri) {
            int m = (li + ri) / 2;
            update(i, j, val, left(ni), li, m);
            update(i, j, val, right(ni), m+1, ri);
        }

        update_row(i, j, val, ni, li, ri);
    }
};
